import * as kysely from 'kysely';
import { Kysely, TableMetadata } from 'kysely';
import { z } from 'zod';
export { session } from './middleware/session.js';
import { Connection, Queue } from 'plainjob';
import * as _conform_to_dom from '@conform-to/dom';
import * as hono_types from 'hono/types';
import * as hono from 'hono';
import 'hono-sessions';
import 'hono/utils/cookie';

declare class Entity<DB, EntityName extends keyof DB & string> {
    private db;
    private name;
    private id;
    private createdAt;
    zod: z.ZodType<DB[EntityName]>;
    constructor(db: Kysely<DB>, name: EntityName, tables: TableMetadata[], _zod?: z.ZodType<DB[EntityName]>, id?: () => string, createdAt?: () => number);
    create(data: Partial<Omit<DB[EntityName], "id" | "createdAt">>): Promise<DB[EntityName]>;
    all<K extends keyof DB[EntityName]>(options?: Partial<Pick<DB[EntityName], K>>, pagination?: {
        limit?: number;
        offset?: number;
    }): Promise<DB[EntityName][]>;
    get(id: string): Promise<DB[EntityName] | undefined>;
    update(id: string, data: Partial<DB[EntityName]>): Promise<DB[EntityName]>;
    delete(id: string): Promise<kysely.DeleteResult[]>;
    private initZodFromTable;
}
type EntityStore<DB> = {
    <T extends keyof DB & string>(key: T): Entity<DB, T>;
    db: Kysely<DB>;
    opts?: EntityStoreOptions<DB>;
};
type EntityStoreOptions<DB> = {
    [K in keyof DB]: {
        id?: () => string;
        createdAt?: () => number;
        zod?: z.ZodType<DB[K]>;
    };
};
declare function store<DB>(db: Kysely<DB>, opts?: EntityStoreOptions<DB>): Promise<EntityStore<DB>>;

declare function migrate(db: Kysely<any>): (...fns: ((db: Kysely<any>) => Promise<void>)[]) => Promise<void>;
/**l
 * Run a function in a database transaction.
 * The transaction is automatically rolled back, even if the function doesn't throw an error.
 * Use during testing, to keep test cases isolated from each other.
 * */
declare function rollback<T>(db: Kysely<T>, fn: (db: Kysely<T>) => Promise<void>): Promise<void>;

declare function test(): boolean;

type Job<T> = {
    name: string;
    run: ({ data }: {
        data: T;
    }) => Promise<void>;
};
declare function job<T>(opts: {
    name: string;
    run: ({ data }: {
        data: T;
    }) => Promise<void>;
}): Job<T>;
type Schedule = {
    name: string;
    cron: string;
    run: () => Promise<void>;
};
declare function queue(opts: {
    connection: Connection;
}): Queue;
declare function work(queue: Queue, jobs: Record<string, Job<any>>, schedules: Record<string, Schedule>): Promise<void>;
declare function perform<T>(queue: Queue, job: Job<T>, data: T): Promise<{
    id: number;
} | {
    id: string;
}>;

type SubmissionPayload<Entry extends FormDataEntryValue> = Entry | SubmissionPayload<Entry>[] | {
    [key: string]: SubmissionPayload<Entry>;
};
type ReplyOptions<FormError> = {
    resetForm?: boolean;
} | {
    formErrors?: FormError;
    fieldErrors?: Record<string, FormError>;
    hideFields?: string[];
};

declare function form<T>(schema: z.ZodSchema<T> | Promise<z.ZodSchema<T>>): hono.MiddlewareHandler<hono.Env, string, {
    in: {
        form: { [K in keyof T]: hono_types.ParsedFormValue | hono_types.ParsedFormValue[]; };
    };
    out: {
        form: {
            status: "success";
            payload: Record<string, SubmissionPayload<FormDataEntryValue>>;
            value: T;
            reply(options?: ReplyOptions<string[]> | undefined): _conform_to_dom.SubmissionResult<string[]>;
        };
    };
}>;

export { form, job, migrate, perform, queue, rollback, store, test, work };
